@@ -1,21 +1,21 @@
 module System.Taffybar.Widget.Text.CPUMonitor (textCpuMonitorNew) where
 
+import Control.Monad.IO.Class ( MonadIO )
 import Text.Printf ( printf )
 import qualified Text.StringTemplate as ST
 import System.Taffybar.Information.CPU
 import System.Taffybar.Widget.Generic.PollingLabel ( pollingLabelNew )
 import qualified GI.Gtk
-import qualified Data.Text as T
 
 -- | Creates a simple textual CPU monitor. It updates once every polling
 -- period (in seconds).
-textCpuMonitorNew :: String -- ^ Format. You can use variables: $total$, $user$, $system$
+textCpuMonitorNew :: MonadIO m
+                  => String -- ^ Format. You can use variables: $total$, $user$, $system$
                   -> Double -- ^ Polling period (in seconds)
-                  -> IO GI.Gtk.Widget
+                  -> m GI.Gtk.Widget
 textCpuMonitorNew fmt period = do
-  label <- pollingLabelNew (T.pack fmt) period callback
-  GI.Gtk.widgetShowAll label
-  return label
+  label <- pollingLabelNew period callback
+  GI.Gtk.toWidget label
   where
     callback = do
       (userLoad, systemLoad, totalLoad) <- cpuLoad
